package org.jboss.odbc;

import java.io.IOException;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * @author mvecera@redhat.com, pmacik@redhat.com
 */
public class ODBCClient {
   private volatile int handle = -1;
   private static SortedSet<Integer> usedHandles = new TreeSet<Integer>();
   private static final Object lock = new Object();
   private static final Object handleLock = new Object();

   public native void connect(String connection) throws IOException;

   public native void execute(String statement) throws IOException;

   public native int getNumCols();

   public native ColumnMetadata getColMetadata(int col);

   public native boolean fetch();

   public native String getColData(int col) throws IOException;

   public native void freeStatement();

   public native void disconnect();

   public native void close();

   private ODBCClient() {
      super();
   }

   public static ODBCClient newInstance() throws IOException {
      ODBCClient c = new ODBCClient();
      synchronized (handleLock) {
         int i = 1;
         while (usedHandles.contains(i) && i <= 1024) {
            i++;
         }
         if (i == 1024) {
            throw new IOException("All handles are currently in use. Try to free some handles by disconnecting from ODBC.");
         }
         usedHandles.add(i);
         c.handle = i;
      }
      return c;
   }

   public void finish() {
      close();
      usedHandles.remove(handle);
   }

   public static class ColumnMetadata {
      public String name;
      public int type;
      public long length;
      public int digits;
      public boolean nullable;

      public ColumnMetadata() {
      }

      public ColumnMetadata(String name, int type, long length, int digits, boolean nullable) {
         this.name = name;
         this.type = type;
         this.length = length;
         this.digits = digits;
         this.nullable = nullable;
      }

      public String toString() {
         return name + ": " + type + "(" + length + ")" + "[" + digits + "]" + (nullable ? "*" : "");
      }
   }
}
