#include <stdio.h>
#include <sql.h>
#include <sqlext.h>
#include "org_jboss_odbc_ODBCClient.h"
#include "util.h"

/*
 * Class:     org_jboss_odbc_ODBCClient
 * Method:    connect
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_org_jboss_odbc_ODBCClient_connect(JNIEnv *jnienv, jobject obj, jstring conn) {
    SQLRETURN ret;
    jbyte *str;
    int h = getStaticHandle(jnienv, obj);
    char error[10240];  
    systemOutPrintln(jnienv, "[%d] connecting 1", h);
    jclass cls = (*jnienv)->GetObjectClass(jnienv, obj);
    systemOutPrintln(jnienv, "[%d] connecting 2", h);
    str = (*jnienv)->GetStringUTFChars(jnienv, conn, NULL);
    systemOutPrintln(jnienv, "[%d] connecting 3", h);
    SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &(env[h]));
    systemOutPrintln(jnienv, "[%d] connecting 4", h);
    SQLSetEnvAttr(env[h], SQL_ATTR_ODBC_VERSION, (void *) SQL_OV_ODBC3, 0);
    systemOutPrintln(jnienv, "[%d] connecting 5", h);
    SQLAllocHandle(SQL_HANDLE_DBC, env[h], &(dbc[h]));
    systemOutPrintln(jnienv, "[%d] connecting 6", h);
    ret = SQLDriverConnect(dbc[h], NULL, str, SQL_NTS, NULL, 0, NULL, SQL_DRIVER_COMPLETE);
    systemOutPrintln(jnienv, "[%d] connecting 7", h);

    if (!SQL_SUCCEEDED(ret)) {
        systemOutPrintln(jnienv, "[%d] connecting 8", h);
        extractError(dbc[h], SQL_HANDLE_DBC, error, sizeof(error));        
        systemOutPrintln(jnienv, "[%d] connecting 9", h);
        throwIOException(jnienv, "SQLDriverConnect failed with return value %d:\n%s", ret, error);
    }
    systemOutPrintln(jnienv, "[%d] connecting 10", h);
    (*jnienv)->ReleaseStringUTFChars(jnienv, conn, str);
    systemOutPrintln(jnienv, "[%d] connecting 11", h);
}

/*
 * Class:     org_jboss_odbc_ODBCClient
 * Method:    execute
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_org_jboss_odbc_ODBCClient_execute(JNIEnv *jnienv, jobject obj, jstring query) {
    SQLRETURN ret;
    char error[10240];
    jbyte *str;
    int h = getStaticHandle(jnienv, obj);
    systemOutPrintln(jnienv, "[%d] executing 1", h);
    str = (*jnienv)->GetStringUTFChars(jnienv, query, NULL);
    systemOutPrintln(jnienv, "[%d] executing 2", h);
    ret = SQLAllocHandle(SQL_HANDLE_STMT, dbc[h], &(stmt[h]));
    systemOutPrintln(jnienv, "[%d] executing 3", h);
    if (!SQL_SUCCEEDED(ret)) {
        systemOutPrintln(jnienv, "[%d] executing 4", h);
        extractError(dbc[h], SQL_HANDLE_DBC, error, sizeof(error));        
        systemOutPrintln(jnienv, "[%d] executing 5", h);
        tdhrowIOException(jnienv, "SQLAllocHandle failed with return value %d:\n%s", ret, error);        
    }
    systemOutPrintln(jnienv, "[%d] executing 6", h);
    ret = SQLExecDirect(stmt[h], str, SQL_NTS);
    systemOutPrintln(jnienv, "[%d] executing 7", h);
    (*jnienv)->ReleaseStringUTFChars(jnienv, query, str);    
    systemOutPrintln(jnienv, "[%d] executing 8", h);
    if (!SQL_SUCCEEDED(ret)) {
        systemOutPrintln(jnienv, "[%d] executing 9", h);
        extractError(stmt[h], SQL_HANDLE_STMT, error, sizeof(error));        
        systemOutPrintln(jnienv, "[%d] executing 10", h);
        throwIOException(jnienv, "SQLExecDirect failed with return value %d:\n%s", ret, error);        
    }
    systemOutPrintln(jnienv, "[%d] executing 11", h);    
}

/*
 * Class:     org_jboss_odbc_ODBCClient
 * Method:    getNumCols
 * Signature: ()J
 */
JNIEXPORT jint JNICALL Java_org_jboss_odbc_ODBCClient_getNumCols(JNIEnv *jnienv, jobject obj) {
    SQLSMALLINT columns;
    int h = getStaticHandle(jnienv, obj);
    
    SQLNumResultCols(stmt[h], &columns);
    
    return (jint) columns;
}

/*
 * Class:     org_jboss_odbc_ODBCClient
 * Method:    fetch
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_jboss_odbc_ODBCClient_fetch(JNIEnv *jnienv, jobject obj) {
    int h = getStaticHandle(jnienv, obj);

    return (jboolean) SQL_SUCCEEDED(SQLFetch(stmt[h]));
}

/*
 * Class:     org_jboss_odbc_ODBCClient
 * Method:    getColData
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_jboss_odbc_ODBCClient_getColData(JNIEnv *jnienv, jobject obj, jint col) {
    int h = getStaticHandle(jnienv, obj);
    SQLINTEGER indicator;
    SQLRETURN ret;
    char buf[5120];
    char error[10240];
       
    ret = SQLGetData(stmt[h], (SQLUSMALLINT) col, SQL_C_CHAR, buf, sizeof(buf), &indicator);
    
    if (SQL_SUCCEEDED(ret)) {
        if (indicator == SQL_NULL_DATA) strcpy(buf, "NULL");
    } else {
        extractError(stmt[h], SQL_HANDLE_STMT, error, sizeof(error));        
        throwIOException(jnienv, "SQLGetData failed with return value %d:\n%s", ret, error);
    }
        
    return (*jnienv)->NewStringUTF(jnienv, buf);
}

/*
 * Class:     org_jboss_odbc_ODBCClient
 * Method:    getColMetadata
 * Signature: (ILorg/jboss/odbc/ODBCClient/ColumnMetadata;)V
 */
JNIEXPORT jobject JNICALL Java_org_jboss_odbc_ODBCClient_getColMetadata(JNIEnv *jnienv, jobject obj, jint col) {
    int h = getStaticHandle(jnienv, obj);
    SQLSMALLINT nameLength, dataType, decimalDigits, nullable;
    SQLULEN colSize;
    jstring jstr;
    char name[512];

    SQLDescribeCol(stmt[h], (SQLUSMALLINT) col, name, sizeof(name), &nameLength, &dataType, &colSize, &decimalDigits, &nullable);
    
    jstr = (*jnienv)->NewStringUTF(jnienv, name);
    
    jclass clazz;
    jmethodID cid;
    jobject meta;

    clazz = (*jnienv)->FindClass(jnienv, "org/jboss/odbc/ODBCClient$ColumnMetadata");
    cid = (*jnienv)->GetMethodID(jnienv, clazz, "<init>", "(Ljava/lang/String;IJIZ)V");
    meta = (*jnienv)->NewObject(jnienv, clazz, cid, jstr, dataType, colSize, decimalDigits, (jboolean) (nullable == SQL_NULLABLE));  

    return meta;
}


/*
 * Class:     org_jboss_odbc_ODBCClient
 * Method:    freeStatement
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_jboss_odbc_ODBCClient_freeStatement(JNIEnv *jnienv, jobject obj) {
    int h = getStaticHandle(jnienv, obj);
    systemOutPrintln(jnienv, "[%d] freeing statement 1", h);
    SQLFreeHandle(SQL_HANDLE_STMT, stmt[h]);
    systemOutPrintln(jnienv, "[%d] freeing statement 2", h);
}

/*
 * Class:     org_jboss_odbc_ODBCClient
 * Method:    close
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_jboss_odbc_ODBCClient_close(JNIEnv *jnienv, jobject obj) {
    int h = getStaticHandle(jnienv, obj);
    systemOutPrintln(jnienv, "[%d] closing 1", h);
    SQLFreeHandle(SQL_HANDLE_ENV, env[h]);   
    systemOutPrintln(jnienv, "[%d] closing 2", h);
}

/*
 * Class:     org_jboss_odbc_ODBCClient
 * Method:    disconnect
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_jboss_odbc_ODBCClient_disconnect(JNIEnv *jnienv, jobject obj) {
    int h = getStaticHandle(jnienv, obj);
    systemOutPrintln(jnienv, "[%d] disconnecting 1", h);
    SQLDisconnect(dbc[h]);
    systemOutPrintln(jnienv, "[%d] disconnecting 2", h);
    SQLFreeHandle(SQL_HANDLE_DBC, dbc[h]);
    systemOutPrintln(jnienv, "[%d] disconnecting 3", h);
}

